@@ -10,14 +10,14 @@ from mezzanine.core.admin import DisplayableAdmin
 
 class PageAdmin(DisplayableAdmin):
     """
-    Admin class for the ``Page`` model and all subclasses of ``Page``. Handles 
-    redirections between admin interfaces for the ``Page`` model and its 
+    Admin class for the ``Page`` model and all subclasses of ``Page``. Handles
+    redirections between admin interfaces for the ``Page`` model and its
     subclasses.
     """
 
     def change_view(self, request, object_id, extra_context=None):
         """
-        For the ``Page`` model, check ``page.get_content_model()`` for a 
+        For the ``Page`` model, check ``page.get_content_model()`` for a
         subclass and redirect to its admin change view.
         """
         if self.model is Page:
@@ -26,14 +26,13 @@ class PageAdmin(DisplayableAdmin):
             if content_model is not None:
                 app = content_model.__class__._meta.app_label
                 name = content_model.__class__.__name__.lower()
-                change_url = reverse("admin:%s_%s_change" % (app, name), 
+                change_url = reverse("admin:%s_%s_change" % (app, name),
                     args=(content_model.id,))
                 return HttpResponseRedirect(change_url)
-        return super(PageAdmin, self).change_view(request, object_id, 
+        return super(PageAdmin, self).change_view(request, object_id,
             extra_context=None)
 
     def changelist_view(self, request, extra_context=None):
-
         """
         Redirect to the ``Page`` changelist view for ``Page`` subclasses.
         """
@@ -45,19 +44,25 @@ class PageAdmin(DisplayableAdmin):
         """
         Set the ID of the parent page if passed in via querystring.
         """
-        obj.parent_id = request.GET.get("parent")
+        # Force parent to be saved to trigger handling of ordering and slugs.
+        parent = request.GET.get("parent")
+        if parent is not None and not change:
+            obj.parent_id = parent
+            obj._order = None
+            obj.slug = None
+            obj.save()
         super(PageAdmin, self).save_model(request, obj, form, change)
-    
+
     def _maintain_parent(self, request, response):
         """
-        Maintain the parent ID in the querystring for response_add and 
+        Maintain the parent ID in the querystring for response_add and
         response_change.
         """
         location = response._headers["location"][1]
         parent = request.GET.get("parent")
         if parent is not None and "?" not in location:
             location += "?parent=%s" % parent
-        return HttpResponseRedirect(location) 
+        return HttpResponseRedirect(location)
 
     def response_add(self, request, obj):
         """
